package com.xkrs.dao;

import com.xkrs.model.entity.FirePointEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Map;

/**
 * @author XinYi Song
 */
@Component
public interface FirePointDao extends JpaRepository<FirePointEntity, Long>, JpaSpecificationExecutor<FirePointEntity> {

    /**
     * 根据火点编码查询火点信息
     */
    FirePointEntity findByFireCode(String fireCode);

    /**
     * 根据火点编码修改火点状态
     */
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE fire_point SET fire_type = ?2 WHERE fire_code = ?1", nativeQuery = true)
    void updateFireTypeByFireCode(String fireCode, String fireType);

    /**
     * 根据火点编码修改审核火点状态
     */
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE fire_point SET audit_fire_type = ?2 WHERE fire_code = ?1", nativeQuery = true)
    void updateAuditFireTypeByFireCode(String fireCode, String auditFireType);

    /**
     * 查询时间段内各植被类型的火点数量
     */
    @Query(value = "select la.land_name landname,count(fp.id) as num from land la left join fire_point fp on fp.land_type = la.land_name AND fp.satellite_time BETWEEN ?1 AND ?2 group by la.land_name", nativeQuery = true)
    List<Map<String, Object>> selectNumByLandType(String startTime, String endTime);

    /**
     * 查询时间段内各地区的火点数量
     */
    @Query(value = "select ci.city_name cityname,count(fp.id) from street ci left join fire_point fp on fp.county_code like concat('%',ci.city_code,'%') AND fp.satellite_time BETWEEN ?1 AND ?2 group by ci.city_name", nativeQuery = true)
    List<Map<String, Object>> selectNumByArea(String startTime, String endTime);

    /**
     * 根据火点编码，修改火点前后图片路径
     */
    @Modifying(clearAutomatically = true)
    @Query(value = "update fire_point set before_fire_image = ?2,after_fire_image = ?3 where fire_code = ?1", nativeQuery = true)
    void updateBeforeFireAndAfterFireImage(String fireCode, String beforeFireImage, String afterFireImage);

    /**
     * 查询街道编码异常的火点
     */
//    @Query(value = "SELECT * FROM fire_point WHERE length(street_code) != 9", nativeQuery = true)
//    update fire_point set street_name = (SELECT street_name from street WHERE fire_point.street_code = street.street_code) WHERE fire_point.street_name ISNULL
    @Query(value = "SELECT * FROM fire_point WHERE street_code ISNULL", nativeQuery = true)
    List<FirePointEntity> selectAbnormalStreetCode();

    @Transactional(rollbackOn = Exception.class)
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE fire_point SET street_code = ?2 WHERE id = ?1", nativeQuery = true)
    void updateStreetCodeById(Integer id, String streetCode);

    @Query(value = "select a.really_name 姓名, a.county_name 所在市, a.telephone 手机号, a.vip_level vip, b.satellite_time 扫描时间, " + "round(cast(b.longitude as numeric) ,6) 经度, round(cast(b.latitude as numeric) ,6) 纬度, b.fire_point_address 详细地址  from sys_user a left join fire_point b on " + "case when left(a.county_code, 4) = '0000' then left(b.county_code, 4) = left(a.county_code, 4) " + "when left(a.county_code, 2) = '00' then left(b.county_code, 2) = left(a.county_code, 2) " + "else left(b.county_code, 2) = left(a.county_code, 2) end where a.vip_level > 0 and satellite_time > ?1", nativeQuery = true)
    List<Map<String, String>> downloadFirePointByVip(String yesterday);

}
